@@ -6,6 +6,8 @@
  */
 
 #include "audex.h"
+#include "models/profilemodel.h"
+#include "utils/schemeparser.h"
 
 /* The heart of audex */
 
@@ -31,14 +33,14 @@ Audex::Audex(QWidget *parent, ProfileModel *profile_model, CDDAModel *cdda_model
         return;
     }
 
-    cdda_extract_thread = new CDDAExtractThread(this, cdda_model->cdio());
+    cdda_extract_thread = new CDDAExtractThread(cdda_model->paranoia());
     if (!cdda_extract_thread) {
         qDebug() << "PANIC ERROR. Could not load object CDDAExtractThread. Low mem?";
         return;
     }
-    cdda_extract_thread->setParanoiaFullMode(Preferences::fullParanoiaMode());
-    cdda_extract_thread->setSkipReadingErrors(Preferences::skipReadingErrors());
-    cdda_extract_thread->setSampleOffset(Preferences::sampleOffset());
+    cdda_extract_thread->enableParanoiaMode(Preferences::fullParanoiaMode());
+    cdda_extract_thread->enableSkipReadErrors(Preferences::skipReadErrors());
+    cdda_extract_thread->setSampleShift(Preferences::sampleShift());
 
     jobs = new AudexJobs();
     connect(jobs, SIGNAL(newJobAvailable()), this, SLOT(start_encode()));
@@ -68,6 +70,9 @@ Audex::Audex(QWidget *parent, ProfileModel *profile_model, CDDAModel *cdda_model
     connect(cdda_extract_thread, SIGNAL(warning(const QString &)), this, SLOT(slot_warning(const QString &)));
     connect(cdda_extract_thread, SIGNAL(error(const QString &, const QString &)), this, SLOT(slot_error(const QString &, const QString &)));
 
+    cdda_extract_thread->setTimestampsIntoLog(
+        profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_LOG_WRITE_TIMESTAMPS_INDEX)).toBool());
+
     process_counter = 0;
     timeout_done = false;
     timeout_counter = 0;
@@ -150,6 +155,10 @@ void Audex::start_extract()
         QString year = cdda_model->year();
         QString genre = cdda_model->genre();
         QString suffix = p_suffix;
+        bool fat32_compatible =
+            profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_FAT32COMPATIBLE_INDEX)).toBool();
+        bool replacespaceswithunderscores =
+            profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_UNDERSCORE_INDEX)).toBool();
         QString basepath = Preferences::basePath();
         int cdnum;
         if (!cdda_model->isMultiCD())
@@ -160,7 +169,18 @@ void Audex::start_extract()
         bool overwrite = Preferences::overwriteExistingFiles();
 
         QString targetFilename;
-        if (!construct_target_filename_for_singlefile(targetFilename, cdnum, nooftracks, artist, title, year, genre, suffix, basepath, overwrite)) {
+        if (!construct_target_filename_for_singlefile(targetFilename,
+                                                      cdnum,
+                                                      nooftracks,
+                                                      artist,
+                                                      title,
+                                                      year,
+                                                      genre,
+                                                      suffix,
+                                                      basepath,
+                                                      overwrite,
+                                                      fat32_compatible,
+                                                      replacespaceswithunderscores)) {
             request_finish(false);
             return;
         }
@@ -208,7 +228,6 @@ void Audex::start_extract()
             QString ttitle = cdda_model->data(cdda_model->index(ex_track_index - 1, CDDA_MODEL_COLUMN_TITLE_INDEX)).toString();
             QString year = cdda_model->year();
             QString genre = cdda_model->genre();
-            QString isrc = cdda_model->cdio()->getISRC(ex_track_index);
             QString suffix = p_suffix;
             QString basepath = Preferences::basePath();
             bool fat32_compatible =
@@ -239,7 +258,7 @@ void Audex::start_extract()
                                                ttitle,
                                                year,
                                                genre,
-                                               isrc,
+                                               QString(),
                                                suffix,
                                                basepath,
                                                fat32_compatible,
@@ -262,7 +281,7 @@ void Audex::start_extract()
                                                ttitle,
                                                year,
                                                genre,
-                                               isrc,
+                                               QString(),
                                                suffix,
                                                basepath,
                                                fat32_compatible,
@@ -363,7 +382,7 @@ void Audex::start_encode()
                                      title,
                                      genre,
                                      year,
-                                     cdda_model->cdio()->getISRC(job->trackNo()),
+                                     QString(),
                                      suffix,
                                      cdda_model->cover(),
                                      tmp_dir.path(),
@@ -416,7 +435,7 @@ void Audex::start_encode()
                                          ttitle,
                                          genre,
                                          year,
-                                         cdda_model->cdio()->getISRC(job->trackNo()),
+                                         QString(),
                                          suffix,
                                          cdda_model->cover(),
                                          tmp_dir.path(),
@@ -435,7 +454,7 @@ void Audex::start_encode()
                                          ttitle,
                                          genre,
                                          year,
-                                         cdda_model->cdio()->getISRC(job->trackNo()),
+                                         QString(),
                                          suffix,
                                          cdda_model->cover(),
                                          tmp_dir.path(),
@@ -472,7 +491,7 @@ void Audex::finish_encode()
 void Audex::calculate_speed_extract()
 {
     if ((last_measuring_point_sector > -1) && (cdda_extract_thread->isProcessing())) {
-        double new_value = (double)(current_sector - last_measuring_point_sector) / (2.0f * (double)SECTORS_PER_SECOND);
+        double new_value = (double)(current_sector - last_measuring_point_sector) / (4.0f * (double)SECTORS_PER_SECOND);
         if (new_value < 0.0f)
             new_value = 0.0f;
         if ((new_value < 0.2f) && (!timeout_done)) {
@@ -509,10 +528,10 @@ void Audex::progress_extract(int percent_of_track, int sector, int overall_secto
         QSet<int> sel = cdda_model->selectedTracks();
         QSet<int>::ConstIterator it(sel.begin()), end(sel.end());
         for (; it != end; ++it) {
-            if ((*it < 0) || (*it > cdda_extract_thread->cdio()->numOfTracks()) || (!cdda_extract_thread->cdio()->isAudioTrack((*it)))) {
+            if ((*it < 0) || (*it > cdda_model->paranoia()->numOfTracks()) || (!cdda_model->paranoia()->isAudioTrack((*it)))) {
                 continue;
             }
-            overall_frames += cdda_extract_thread->cdio()->numOfFramesOfTrack((*it));
+            overall_frames += cdda_model->paranoia()->numOfFramesOfTrack((*it));
         }
     }
 
@@ -590,24 +609,25 @@ bool Audex::construct_target_filename(QString &targetFilename,
     Q_UNUSED(is_first_track);
 
     SchemeParser schemeparser;
-    targetFilename = ((basepath.right(1) == "/") ? basepath : basepath + "/")
-        + schemeparser.parsePerTrackFilenameScheme(
-            profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_SCHEME_INDEX)).toString(),
-            trackno,
-            cdno,
-            gindex,
-            nooftracks,
-            artist,
-            title,
-            tartist,
-            ttitle,
-            year,
-            genre,
-            isrc,
-            ext,
-            fat32_compatible,
-            replacespaceswithunderscores,
-            _2digitstracknum);
+    QString basename = schemeparser.parsePerTrackFilenameScheme(
+        profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_SCHEME_INDEX)).toString(),
+        trackno,
+        cdno,
+        gindex,
+        nooftracks,
+        defuse_for_filename(artist),
+        defuse_for_filename(title),
+        defuse_for_filename(tartist),
+        defuse_for_filename(ttitle),
+        defuse_for_filename(year),
+        defuse_for_filename(genre),
+        isrc,
+        ext,
+        fat32_compatible,
+        replacespaceswithunderscores,
+        _2digitstracknum);
+
+    targetFilename = ((basepath.right(1) == "/") ? basepath : basepath + "/") + basename;
 
     int lastSlash = targetFilename.lastIndexOf("/", -1);
     if (lastSlash == -1) {
@@ -654,6 +674,8 @@ bool Audex::construct_target_filename_for_singlefile(QString &targetFilename,
                                                      const QString &genre,
                                                      const QString &ext,
                                                      const QString &basepath,
+                                                     bool fat32_compatible,
+                                                     bool replacespaceswithunderscores,
                                                      bool overwrite_existing_files)
 {
     SchemeParser schemeparser;
@@ -662,12 +684,13 @@ bool Audex::construct_target_filename_for_singlefile(QString &targetFilename,
             profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_SF_NAME_INDEX)).toString(),
             cdno,
             nooftracks,
-            artist,
-            title,
-            date,
-            genre,
+            defuse_for_filename(artist),
+            defuse_for_filename(title),
+            defuse_for_filename(date),
+            defuse_for_filename(genre),
             ext,
-            false);
+            fat32_compatible,
+            replacespaceswithunderscores);
 
     int lastSlash = targetFilename.lastIndexOf("/", -1);
     if (lastSlash == -1) {
@@ -790,16 +813,20 @@ void Audex::execute_finish()
             QString format = profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_SC_FORMAT_INDEX)).toString();
 
             SchemeParser schemeparser;
-            QString filename = schemeparser.parseFilenameScheme(
-                scheme,
-                cdda_model->cdNum(),
-                cdda_model->numOfAudioTracks(),
-                cdda_model->artist(),
-                cdda_model->title(),
-                QString("%1").arg(cdda_model->year()),
-                cdda_model->genre(),
-                format.toLower(),
-                profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_FAT32COMPATIBLE_INDEX)).toBool());
+            QString filename = schemeparser.parseFilenameScheme(scheme,
+                                                                cdda_model->cdNum(),
+                                                                cdda_model->numOfAudioTracks(),
+                                                                defuse_for_filename(cdda_model->artist()),
+                                                                defuse_for_filename(cdda_model->title()),
+                                                                QString("%1").arg(cdda_model->year()),
+                                                                defuse_for_filename(cdda_model->genre()),
+                                                                format.toLower());
+
+            if (profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_FAT32COMPATIBLE_INDEX)).toBool())
+                filename = SchemeParser::makeFAT32FilenameCompatible(filename);
+
+            if (profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_UNDERSCORE_INDEX)).toBool())
+                filename = SchemeParser::replaceSpacesWithUnderscores(filename);
 
             if (p_prepare_dir(filename, target_dir, overwrite)) {
                 if (image.save(filename, format.toLatin1().data())) {
@@ -825,16 +852,20 @@ void Audex::execute_finish()
         bool is_utf8 = profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_PL_UTF8_INDEX)).toBool();
 
         SchemeParser schemeparser;
-        QString filename = schemeparser.parseFilenameScheme(
-            scheme,
-            cdda_model->cdNum(),
-            cdda_model->numOfAudioTracks(),
-            cdda_model->artist(),
-            cdda_model->title(),
-            QString("%1").arg(cdda_model->year()),
-            cdda_model->genre(),
-            format.toLower(),
-            profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_FAT32COMPATIBLE_INDEX)).toBool());
+        QString filename = schemeparser.parseFilenameScheme(scheme,
+                                                            cdda_model->cdNum(),
+                                                            cdda_model->numOfAudioTracks(),
+                                                            defuse_for_filename(cdda_model->artist()),
+                                                            defuse_for_filename(cdda_model->title()),
+                                                            QString("%1").arg(cdda_model->year()),
+                                                            defuse_for_filename(cdda_model->genre()),
+                                                            format.toLower());
+
+        if (profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_FAT32COMPATIBLE_INDEX)).toBool())
+            filename = SchemeParser::makeFAT32FilenameCompatible(filename);
+
+        if (profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_UNDERSCORE_INDEX)).toBool())
+            filename = SchemeParser::replaceSpacesWithUnderscores(filename);
 
         if (p_prepare_dir(filename, target_dir, (overwrite && !cdda_model->isMultiCD() && (cdda_model->cdNum() < 1)))) {
             QFile file(filename);
@@ -892,12 +923,17 @@ void Audex::execute_finish()
             profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_INF_NAME_INDEX)).toString(),
             cdda_model->cdNum(),
             cdda_model->numOfAudioTracks(),
-            cdda_model->artist(),
-            cdda_model->title(),
+            defuse_for_filename(cdda_model->artist()),
+            defuse_for_filename(cdda_model->title()),
             QString("%1").arg(cdda_model->year()),
-            cdda_model->genre(),
-            profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_INF_SUFFIX_INDEX)).toString(),
-            profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_FAT32COMPATIBLE_INDEX)).toBool());
+            defuse_for_filename(cdda_model->genre()),
+            profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_INF_SUFFIX_INDEX)).toString());
+
+        if (profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_FAT32COMPATIBLE_INDEX)).toBool())
+            filename = SchemeParser::makeFAT32FilenameCompatible(filename);
+
+        if (profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_UNDERSCORE_INDEX)).toBool())
+            filename = SchemeParser::replaceSpacesWithUnderscores(filename);
 
         if (p_prepare_dir(filename, target_dir, overwrite)) {
             QFile file(filename);
@@ -910,7 +946,7 @@ void Audex::execute_finish()
                                                  cdda_model->title(),
                                                  QString("%1").arg(cdda_model->year()),
                                                  cdda_model->genre(),
-                                                 cdda_model->cdio()->getMCN(),
+                                                 QString(),
                                                  DiscIDCalculator::CDDBId(cdda_model->discSignature()),
                                                  p_size_of_all_files(target_filename_list),
                                                  cdda_model->lengthOfAudioTracksInSelection(),
@@ -936,12 +972,13 @@ void Audex::execute_finish()
             scheme,
             cdda_model->cdNum(),
             cdda_model->numOfAudioTracks(),
-            cdda_model->artist(),
-            cdda_model->title(),
+            defuse_for_filename(cdda_model->artist()),
+            defuse_for_filename(cdda_model->title()),
             QString("%1").arg(cdda_model->year()),
-            cdda_model->genre(),
+            defuse_for_filename(cdda_model->genre()),
             format.toLower(),
-            profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_FAT32COMPATIBLE_INDEX)).toBool());
+            profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_FAT32COMPATIBLE_INDEX)).toBool(),
+            profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_UNDERSCORE_INDEX)).toBool());
 
         if (p_prepare_dir(filename, target_dir, (overwrite && !cdda_model->isMultiCD() && (cdda_model->cdNum() < 1)))) {
             QFile file(filename);
@@ -976,20 +1013,21 @@ void Audex::execute_finish()
     if ((p_finished_successful) && (profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_CUE_INDEX)).toBool())
         && (((target_filename_list.count() > 0) && !p_single_file) || p_single_file)) {
         QString scheme = profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_CUE_NAME_INDEX)).toString();
-        bool writeMCNAndISRC =
-            profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_CUE_ADD_MCN_AND_ISRC_INDEX)).toBool();
+        // bool writeMCNAndISRC =
+        //     profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_CUE_ADD_MCN_AND_ISRC_INDEX)).toBool();
 
         SchemeParser schemeparser;
         QString filename = schemeparser.parseFilenameScheme(
             scheme,
             cdda_model->cdNum(),
             cdda_model->numOfAudioTracks(),
-            cdda_model->artist(),
-            cdda_model->title(),
+            defuse_for_filename(cdda_model->artist()),
+            defuse_for_filename(cdda_model->title()),
             QString("%1").arg(cdda_model->year()),
-            cdda_model->genre(),
+            defuse_for_filename(cdda_model->genre()),
             "cue",
-            profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_FAT32COMPATIBLE_INDEX)).toBool());
+            profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_FAT32COMPATIBLE_INDEX)).toBool(),
+            profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_UNDERSCORE_INDEX)).toBool());
 
         if (p_prepare_dir(filename, target_dir, overwrite)) {
             QFile file(filename);
@@ -998,11 +1036,9 @@ void Audex::execute_finish()
                 QTextStream out(&file);
                 CueSheetWriter cuesheetwriter(cdda_model);
                 if (p_single_file) {
-                    out << cuesheetwriter.cueSheet(target_single_filename, Preferences::sampleOffset() / CD_FRAMESIZE_SAMPLES, writeMCNAndISRC, writeMCNAndISRC)
-                               .join("\n");
+                    out << cuesheetwriter.cueSheet(target_single_filename, Preferences::sampleShift() / CD_FRAMESIZE_SAMPLES).join("\n");
                 } else {
-                    out << cuesheetwriter.cueSheet(target_filename_list, Preferences::sampleOffset() / CD_FRAMESIZE_SAMPLES, writeMCNAndISRC, writeMCNAndISRC)
-                               .join("\n");
+                    out << cuesheetwriter.cueSheet(target_filename_list, Preferences::sampleShift() / CD_FRAMESIZE_SAMPLES).join("\n");
                 }
                 file.close();
                 Q_EMIT info(i18n("Cue sheet \"%1\" successfully created.", QFileInfo(filename).fileName()));
@@ -1020,12 +1056,13 @@ void Audex::execute_finish()
             profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_LOG_NAME_INDEX)).toString(),
             cdda_model->cdNum(),
             cdda_model->numOfAudioTracks(),
-            cdda_model->artist(),
-            cdda_model->title(),
+            defuse_for_filename(cdda_model->artist()),
+            defuse_for_filename(cdda_model->title()),
             QString("%1").arg(cdda_model->year()),
-            cdda_model->genre(),
+            defuse_for_filename(cdda_model->genre()),
             "log",
-            profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_FAT32COMPATIBLE_INDEX)).toBool());
+            profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_FAT32COMPATIBLE_INDEX)).toBool(),
+            profile_model->data(profile_model->index(profile_model->currentProfileRow(), PROFILE_MODEL_COLUMN_UNDERSCORE_INDEX)).toBool());
 
         if (p_prepare_dir(filename, target_dir, overwrite)) {
             QFile file(filename);
