@@ -32,14 +32,6 @@ class deviceWidget : public deviceWidgetUI
 public:
     explicit deviceWidget(QWidget *parent = nullptr);
     ~deviceWidget() override;
-public Q_SLOTS:
-    void setDeviceInfo(const QString &vendor,
-                       const QString &model,
-                       const QString &revision,
-                       const bool canReadMCN,
-                       const bool canReadISRC,
-                       const bool c2ErrorCorrection);
-    void clearDeviceInfo();
 };
 
 #endif
